@@ -3,6 +3,8 @@
  *
  * Author:
  *    Michael Meeks (michael@imaginator.com)
+ *
+ * (C) 1998, 1999 Michael Meeks
  **/
 #ifndef GNUMERIC_EXCEL_H
 #define GNUMERIC_EXCEL_H
@@ -23,7 +25,7 @@ extern int      ms_excel_write_workbook (MsOle *file, Workbook *wb,
 
 /* We need to use these for both read and write */
 typedef struct {
-	int b, g, r;
+	int r, g, b;
 } EXCEL_PALETTE_ENTRY;
 extern  EXCEL_PALETTE_ENTRY const excel_default_palette[];
 #define EXCEL_DEF_PAL_LEN   56
