@@ -46,7 +46,9 @@ typedef struct
 /* Cell contains a comment */
 #define CELL_HAS_COMMENT       1
 
-#define CELL_FORMAT_SET        2
+/*
+ * #define CELL_FORMAT_SET        2
+ */
 
 /* Cell has been queued for recalc */
 #define CELL_QUEUED_FOR_RECALC 4
@@ -83,7 +85,6 @@ struct _Cell {
 	/* Type of the content and the actual parsed content */
 	ExprTree    *parsed_node;	/* Parse tree with the expression */
 	Value       *value;		/* Last value computed */
-	Style       *style;		/* The Cell's style */
 
 	StyleColor  *render_color;      /* If a manually entered color has been selected */
 
@@ -97,7 +98,9 @@ struct _Cell {
 	char        generation;
 };
 
-#define CELL_IS_FORMAT_SET(cell) ((cell)->flags & CELL_FORMAT_SET)
+/*
+ * #define CELL_IS_FORMAT_SET(cell) ((cell)->flags & CELL_FORMAT_SET)
+ */
 
 typedef enum {
 	CELL_COPY_TYPE_CELL,
@@ -105,10 +108,13 @@ typedef enum {
 } CellCopyType;
 
 typedef struct {
-	int  col_offset, row_offset; /* Position of the cell */
-	int  type;
+	int col_offset, row_offset; /* Position of the cell */
+	guint8 type;
 	union {
-		Cell *cell;
+		struct {
+			Cell   *cell;
+			MStyle *mstyle;
+		} cell;
 		char *text;
 	} u;
 } CellCopy;
@@ -136,27 +142,17 @@ void        cell_set_array_formula       (Sheet *sheet, int rowa, int cola,
 void        cell_set_format              (Cell *cell, const char *format);
 void        cell_set_format_simple       (Cell *cell, const char *format);
 void        cell_set_format_from_style   (Cell *cell, StyleFormat *style_format);
-void        cell_set_style               (Cell *cell, Style *reference_style);
 void        cell_set_comment             (Cell *cell, const char *str);
 void        cell_comment_destroy         (Cell *cell);
 void        cell_comment_reposition      (Cell *cell);
-void        cell_set_font_from_style     (Cell *cell, StyleFont *style_font);
 char       *cell_get_comment             (Cell *cell);
-void        cell_set_foreground          (Cell *cell, gushort red,
-					  gushort green, gushort blue);
-void        cell_set_background          (Cell *cell, gushort red,
-					  gushort green, gushort blue);
-void        cell_set_color_from_style    (Cell *cell, StyleColor *foreground, 
-					  StyleColor *background);
-void        cell_set_pattern             (Cell *cell, int pattern);
-void        cell_set_border              (Cell *cell,
-					  StyleBorderType const border_type [4],
-					  StyleColor *border_color [4]);
-void        cell_set_alignment           (Cell *cell, int halign, int valign,
-					  int orientation, int auto_return);
-void        cell_set_halign              (Cell *cell, StyleHAlignFlags halign);
+
 void        cell_set_rendered_text       (Cell *cell, const char *rendered_text);
+MStyle     *cell_get_mstyle              (const Cell *cell);
+void        cell_set_mstyle              (const Cell *cell, MStyle *mstyle);
+void        cell_style_changed           (Cell *cell);
 void        cell_relocate                (Cell *cell);
+
 void        cell_get_span                (Cell *cell, int *col1, int *col2);
 char       *cell_get_text                (Cell *cell);
 char       *cell_get_content             (Cell *cell);
@@ -167,7 +163,7 @@ void        cell_calc_dimensions         (Cell *cell);
 Cell       *cell_copy                    (const Cell *cell);
 void        cell_destroy                 (Cell *cell);
 void        cell_queue_redraw            (Cell *cell);
-int         cell_get_horizontal_align    (const Cell *cell);
+int         cell_get_horizontal_align    (const Cell *cell, int align);
 int	    cell_is_number  		 (const Cell *cell);
 
 int         cell_draw                    (Cell *cell, SheetView *sheet_view,
