@@ -3,10 +3,6 @@
  *
  * Author:
  *  Miguel de Icaza (miguel@gnu.org)
- *
- * FIXME:
- *    I would like to have a global selection instead of the per-sheet/workbook
- *    clipboard, it would clean up a few code spots.
  */
 #include <config.h>
 #include <gnome.h>
@@ -17,6 +13,7 @@
 #include "clipboard.h"
 #include "eval.h"
 #include "selection.h"
+#include "application.h"
 #include "render-ascii.h"
 
 /*
@@ -82,18 +79,33 @@ paste_cell_flags (Sheet *dest_sheet, int target_col, int target_row,
 		  CellCopy *c_copy, int paste_flags)
 {
 	if (!(paste_flags & (PASTE_FORMULAS | PASTE_VALUES))){
-		Cell *cell;
-		
-		cell = sheet_cell_get (dest_sheet,
-				       target_col,
-				       target_row);
-		if (cell && c_copy->u.cell)
-			cell_set_style (cell, c_copy->u.cell->style);
+		Range r;
+
+		r.start.col = target_col;
+		r.start.row = target_row;
+		r.end.col   = target_col;
+		r.end.row   = target_row;
+		if (c_copy->u.cell.mstyle) {
+			mstyle_ref (c_copy->u.cell.mstyle);
+			sheet_style_attach (dest_sheet, r, c_copy->u.cell.mstyle);
+		}
 	} else {
 		Cell *new_cell;
 		
-		if (c_copy->type != CELL_COPY_TYPE_TEXT){
-			new_cell = cell_copy (c_copy->u.cell);
+		if (c_copy->type != CELL_COPY_TYPE_TEXT) {
+			Range r;
+
+			new_cell = cell_copy (c_copy->u.cell.cell);
+			
+			r.start.col = target_col;
+			r.start.row = target_row;
+			r.end.col   = target_col;
+			r.end.row   = target_row;
+			if (c_copy->u.cell.mstyle) {
+				mstyle_ref (c_copy->u.cell.mstyle);
+				sheet_style_attach (dest_sheet, r,
+						    c_copy->u.cell.mstyle);
+			}
 			
 			return paste_cell (
 				dest_sheet, new_cell,
@@ -145,7 +157,7 @@ do_clipboard_paste_cell_region (CellRegion *region, Sheet *dest_sheet,
 					  dest_row + paste_height - 1);
 	
 	/* Paste each element */
-	if (paste_flags & PASTE_TRANSPOSE){
+	if (paste_flags & PASTE_TRANSPOSE) {
 		col_inc = region->rows;
 		row_inc = region->cols;
 	} else {
@@ -186,7 +198,7 @@ do_clipboard_paste_cell_region (CellRegion *region, Sheet *dest_sheet,
 		dest_col + paste_width - 1,
 		dest_row + paste_height -1);
 
-	if (deps){
+	if (deps) {
 		cell_queue_recalc_list (deps, TRUE);
 		formulas = 1;
 	}
@@ -222,7 +234,7 @@ new_node (GList *list, char *data, char *p, int col, int row)
 /**
  * x_selection_to_cell_region:
  * @data: points to an array of chars are received.
- * @len:  The lenght of the @data buffer as received. 
+ * @len:  The length of the @data buffer as received. 
  *
  * Creates a CellRegion based on the X selection
  *
@@ -257,7 +269,7 @@ x_selection_to_cell_region (char *data, int len)
 	}
 
 	/* Handle the remainings */
-	if (p != data){
+	if (p != data) {
 		list = new_node (list, data, p, cur_col, rows);
 		cur_col++;
 		if (cur_col > cols)
@@ -321,6 +333,15 @@ sheet_paste_selection (Sheet *sheet, CellRegion *content, SheetSelection *ss, cl
 		paste_width,  paste_height,
 		pc->paste_flags);
 
+	{
+		Range r;
+		r.start.col = pc->dest_col;
+		r.end.col   = pc->dest_col + paste_width;
+		r.start.row = pc->dest_row;
+		r.end.row   = pc->dest_row + paste_height;
+		sheet_style_optimize (sheet, r);
+	}
+
 	sheet_cursor_set (pc->dest_sheet,
 			  pc->dest_col, pc->dest_row,
 			  pc->dest_col, pc->dest_row,
@@ -377,17 +398,31 @@ x_selection_received (GtkWidget *widget, GtkSelectionData *sel, guint time, gpoi
 static void
 x_selection_handler (GtkWidget *widget, GtkSelectionData *selection_data, guint info, guint time, gpointer data)
 {
-	Workbook *wb = (Workbook *) data;
+	gboolean content_needs_free = FALSE;
+	CellRegion *clipboard = application_clipboard_contents_get ();
 	char *rendered_selection;
 	
-	g_assert (wb->clipboard_contents);
+	if (clipboard == NULL) {
+		Sheet *sheet = application_clipboard_sheet_get ();
+		Range const *a = application_clipboard_area_get ();
+		content_needs_free = TRUE;
+		clipboard =
+		    clipboard_copy_cell_range (sheet,
+					       a->start.col, a->start.row,
+					       a->end.col,   a->end.row);
+	}
 
-	rendered_selection = cell_region_render_ascii (wb->clipboard_contents);
+	g_return_if_fail (clipboard != NULL);
+
+	rendered_selection = cell_region_render_ascii (clipboard);
 	
 	gtk_selection_data_set (
 		selection_data, GDK_SELECTION_TYPE_STRING, 8,
 		rendered_selection, strlen (rendered_selection));
 	g_free (rendered_selection);
+
+	if (content_needs_free)
+		clipboard_release (clipboard);
 }
 
 /**
@@ -417,15 +452,15 @@ x_clipboard_bind_workbook (Workbook *wb)
 	
 	gtk_signal_connect (
 		GTK_OBJECT (wb->toplevel), "selection_clear_event",
-		GTK_SIGNAL_FUNC(x_selection_clear), wb);
+		GTK_SIGNAL_FUNC (x_selection_clear), wb);
 
 	gtk_signal_connect (
 		GTK_OBJECT (wb->toplevel), "selection_received",
-		GTK_SIGNAL_FUNC(x_selection_received), wb);
+		GTK_SIGNAL_FUNC (x_selection_received), wb);
 
 	gtk_signal_connect (
 		GTK_OBJECT (wb->toplevel), "selection_get",
-		GTK_SIGNAL_FUNC(x_selection_handler), wb);
+		GTK_SIGNAL_FUNC (x_selection_handler), NULL);
 
 	gtk_selection_add_target (
 		wb->toplevel,
@@ -461,9 +496,11 @@ clipboard_prepend_cell (Sheet *sheet, int col, int row, Cell *cell, void *user_d
 	copy = g_new (CellCopy, 1);
 
 	copy->type = CELL_COPY_TYPE_CELL;
-	copy->u.cell = cell_copy (cell);
-	copy->col_offset  = col - c->base_col;
-	copy->row_offset  = row - c->base_row;
+	copy->u.cell.cell = cell_copy (cell);
+	/* Horrific inefficiency */
+	copy->u.cell.mstyle = sheet_style_compute (sheet, col, row);
+	copy->col_offset    = col - c->base_col;
+	copy->row_offset    = row - c->base_row;
 	
 	c->r->list = g_list_prepend (c->r->list, copy);
 
@@ -615,10 +652,12 @@ clipboard_release (CellRegion *region)
 		
 		if (this_cell->type != CELL_COPY_TYPE_TEXT) {
 			/* The cell is not really in the rows or columns */
-			this_cell->u.cell->sheet = NULL;
-			this_cell->u.cell->row = NULL;
-			this_cell->u.cell->col = NULL;
-			cell_destroy (this_cell->u.cell);
+			this_cell->u.cell.cell->sheet = NULL;
+			this_cell->u.cell.cell->row = NULL;
+			this_cell->u.cell.cell->col = NULL;
+			mstyle_unref (this_cell->u.cell.mstyle);
+			this_cell->u.cell.mstyle = NULL;
+			cell_destroy (this_cell->u.cell.cell);
 		} else
 			g_free (this_cell->u.text);
 		g_free (this_cell);
