@@ -1254,6 +1254,87 @@ gnumeric_rate (FunctionEvalInfo *ei, Value **argv)
 
 /***************************************************************************/
 
+static char *help_irr = {
+	N_("@FUNCTION=IRR\n"
+	   "@SYNTAX=IRR(values[,guess])\n"
+	   "@DESCRIPTION="
+	   "IRR calculates and returns the internal rate of return of an "
+	   "investment.  This function is closely related to the net present "
+	   "value function (NPV).  The IRR is the interest rate for a "
+	   "serie of cash flow where the net preset value is zero. "
+	   "\n"
+	   "@values contains the serie of cash flow generated by the "
+	   "investment.  The payments should occur at regular intervals.  "
+	   "The optional @guess is the initial value used in calculating "
+	   "the IRR.  You do not have to use that, it is only provided "
+	   "for the Excel compatibility. "
+	   "\n"
+	   "This function is Excel compatible. "
+	   "\n"
+	   "@EXAMPLES=\n"
+	   "Let us assume that the cells A1:A8 contain the numbers -32432, "
+	   "5324, 7432, 9332, 12324, 4334, 1235, -3422.  Now\n"
+	   "IRR(A1:A8) returns 0.04375. "
+	   "\n"
+	   "@SEEALSO=FV,NPV,PV")
+};
+
+typedef struct {
+        int     n;
+        float_t *values;
+} gnumeric_irr_t;
+
+static GoalSeekStatus
+irr_npv (float_t rate, float_t *y, void *user_data)
+{
+	gnumeric_irr_t *p = user_data;
+	float_t        *values, sum;
+        int            i, n;
+
+	values = p->values;
+	n = p->n;
+
+	sum = 0;
+	for (i=0; i<n; i++)
+	        sum += values[i] / pow(1 + rate, i);
+
+	*y = sum;
+	return GOAL_SEEK_OK;
+}
+
+static Value *
+gnumeric_irr (FunctionEvalInfo *ei, Value **argv)
+{
+	GoalSeekData    data;
+	GoalSeekStatus  status;
+	Value           *result = NULL;
+	gnumeric_irr_t  p;
+	float_t         rate0;
+	int             n;
+
+	goal_seek_initialise (&data);
+	rate0 = 0.1; /* Ignore the guess value */
+
+	p.values = collect_floats_value (argv[0], &ei->pos,
+					 COLLECT_IGNORE_STRINGS,
+					 &n, &result);
+	if (result != NULL) {
+		g_free (p.values);
+	        return result;
+	}
+
+	p.n = n;
+	status = goal_seek_newton (&irr_npv, NULL, &data, &p, rate0);
+	g_free (p.values);
+
+	if (status == GOAL_SEEK_OK)
+		return value_new_float (data.root);
+	else
+		return value_new_error (&ei->pos, gnumeric_err_NUM);
+}
+
+/***************************************************************************/
+
 static char *help_pv = {
 	N_("@FUNCTION=PV\n"
 	   "@SYNTAX=PV(rate,nper,pmt,fv,type)\n"
@@ -1402,6 +1483,110 @@ gnumeric_xnpv (FunctionEvalInfo *ei, Value **argv)
 
 /***************************************************************************/
 
+static char *help_xirr = {
+	N_("@FUNCTION=XIRR\n"
+	   "@SYNTAX=XIRR(values,dates[,guess])\n"
+	   "@DESCRIPTION="
+	   "XIRR calculates and returns the internal rate of return of an "
+	   "investment that has not necessarily periodic payments.  This "
+	   "function is closely related to the net present value function "
+	   "(NPV and XNPV).  The XIRR is the interest rate for a "
+	   "serie of cash flow where the XNPV is zero. "
+	   "\n"
+	   "@values contains the serie of cash flow generated by the "
+	   "investment.  @dates contains the dates of the payments.  The "
+	   "first date describes the payment day of the initial payment and "
+	   "thus all the other dates should be after this date. "
+	   "The optional @guess is the initial value used in calculating "
+	   "the XIRR.  You do not have to use that, it is only provided "
+	   "for the Excel compatibility. "
+	   "\n"
+	   "This function is Excel compatible. "
+	   "\n"
+	   "@EXAMPLES=\n"
+	   "Let us assume that the cells A1:A5 contain the numbers -6000, "
+	   "2134, 1422, 1933, and 1422, and the cells B1:B5 contain the "
+	   "dates \"1999-01-15\", \"1999-04-04\", \"1999-05-09\", "
+	   "\"2000-03-12\", and \"2000-05-1\". Then "
+	   "XIRR(A1:A5,B1:B5) returns 0.224838. "
+	   "\n"
+	   "@SEEALSO=IRR,XNPV")
+};
+
+typedef struct {
+        int     n;
+        float_t *values;
+        float_t *dates;
+} gnumeric_xirr_t;
+
+static GoalSeekStatus
+xirr_npv (float_t rate, float_t *y, void *user_data)
+{
+	gnumeric_xirr_t *p = user_data;
+	float_t         *values, *dates, sum;
+        int             i, n;
+
+	values = p->values;
+	dates = p->dates;
+	n = p->n;
+
+	sum = 0;
+	for (i=0; i<n; i++) {
+	        float_t d = dates[i] - dates[0];
+
+		if (d < 0)
+		        return GOAL_SEEK_ERROR;
+	        sum += values[i] / pow(1+rate, d/365.0);
+	}
+
+	*y = sum;
+	return GOAL_SEEK_OK;
+}
+
+static Value *
+gnumeric_xirr (FunctionEvalInfo *ei, Value **argv)
+{
+	GoalSeekData    data;
+	GoalSeekStatus  status;
+	Value           *result = NULL;
+	gnumeric_xirr_t p;
+	float_t         rate0;
+	int             n, d_n;
+
+	goal_seek_initialise (&data);
+	rate0 = 0.1; /* Ignore the guess value */
+
+	p.values = collect_floats_value (argv[0], &ei->pos,
+					 COLLECT_IGNORE_STRINGS,
+					 &n, &result);
+	p.dates = NULL;
+
+	if (result != NULL)
+		goto out;
+
+	p.dates = collect_floats_value (argv[1], &ei->pos,
+					COLLECT_DATES,
+					&d_n, &result);
+	if (result != NULL)
+		goto out;
+
+	p.n = n;
+	status = goal_seek_newton (&xirr_npv, NULL, &data, &p, rate0);
+
+	if (status == GOAL_SEEK_OK)
+		result = value_new_float (data.root);
+	else
+		result = value_new_error (&ei->pos, gnumeric_err_NUM);
+
+ out:
+	g_free (p.values);
+	g_free (p.dates);
+
+	return result;
+}
+
+/***************************************************************************/
+
 static char *help_fv = {
 	N_("@FUNCTION=FV\n"
 	   "@SYNTAX=FV(rate,nper,pmt,pv,type)\n"
@@ -1733,6 +1918,9 @@ finance_functions_init (void)
 			    &help_intrate,  gnumeric_intrate);
 	function_add_args  (cat, "ipmt", "ffff|ff", "rate,per,nper,pv,fv,type",
 			    &help_ipmt,     gnumeric_ipmt);
+	function_add_args  (cat, "irr", "A|f",
+			    "values[,guess]",
+			    &help_irr,      gnumeric_irr);
 	function_add_args  (cat, "ispmt", "ffff",    "rate,per,nper,pv",
 			    &help_ispmt,  gnumeric_ispmt);
 	function_add_args  (cat, "mirr", "Aff",
@@ -1777,6 +1965,8 @@ finance_functions_init (void)
         function_add_args  (cat, "tbillyield", "??f",
 			    "settlement,maturity,pr",
 			    &help_tbillyield,  gnumeric_tbillyield);
+        function_add_args  (cat, "xirr", "AA|f", "values,dates[,guess]",
+			    &help_xirr,     gnumeric_xirr);
         function_add_args  (cat, "xnpv", "fAA", "rate,values,dates",
 			    &help_xnpv,     gnumeric_xnpv);
 }
