@@ -15,6 +15,7 @@
 #include "utils.h"
 #include "gnumeric-util.h"
 #include "selection.h"
+#include "workbook-cmd-format.h"
 
 /* Marshal forward declarations */
 static void   item_bar_marshal      (GtkObject *,
@@ -58,6 +59,33 @@ item_bar_destroy (GtkObject *object)
 		(*GTK_OBJECT_CLASS (item_bar_parent_class)->destroy)(object);
 }
 
+static void
+item_bar_fonts_unref (ItemBar *item_bar)
+{
+	if (item_bar->normal_font != NULL) {
+		style_font_unref (item_bar->normal_font);
+		item_bar->normal_font = NULL;
+	}
+
+	if (item_bar->bold_font != NULL) {
+		style_font_unref (item_bar->bold_font);
+		item_bar->bold_font = NULL;
+	}
+}
+
+void
+item_bar_fonts_init (ItemBar *item_bar, double const zoom_factor)
+{
+	item_bar_fonts_unref (item_bar);
+
+	item_bar->normal_font =
+		style_font_new_simple (DEFAULT_FONT, DEFAULT_SIZE,
+				       zoom_factor, FALSE, FALSE);
+	item_bar->bold_font =
+		style_font_new_simple (DEFAULT_FONT, DEFAULT_SIZE,
+				       zoom_factor, TRUE, FALSE);
+}
+
 static void
 item_bar_realize (GnomeCanvasItem *item)
 {
@@ -83,8 +111,7 @@ item_bar_realize (GnomeCanvasItem *item)
 	else
 		item_bar->change_cursor = gdk_cursor_new (GDK_SB_H_DOUBLE_ARROW);
 
-	/* Reference the bold font */
-	style_font_ref (gnumeric_default_bold_font);
+	item_bar_fonts_init (item_bar, 1.);
 }
 
 static void
@@ -92,10 +119,10 @@ item_bar_unrealize (GnomeCanvasItem *item)
 {
 	ItemBar *item_bar = ITEM_BAR (item);
 
-	style_font_unref (gnumeric_default_bold_font);
 	gdk_gc_unref (item_bar->gc);
 	gdk_cursor_destroy (item_bar->change_cursor);
 	gdk_cursor_destroy (item_bar->normal_cursor);
+	item_bar_fonts_unref (item_bar);
 
 	if (GNOME_CANVAS_ITEM_CLASS (item_bar_parent_class)->unrealize)
 		(*GNOME_CANVAS_ITEM_CLASS (item_bar_parent_class)->unrealize)(item);
@@ -104,8 +131,8 @@ item_bar_unrealize (GnomeCanvasItem *item)
 static void
 item_bar_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags)
 {
-	if (GNOME_CANVAS_ITEM_CLASS(item_bar_parent_class)->update)
-		(*GNOME_CANVAS_ITEM_CLASS(item_bar_parent_class)->update)(item, affine, clip_path, flags);
+	if (GNOME_CANVAS_ITEM_CLASS (item_bar_parent_class)->update)
+		(*GNOME_CANVAS_ITEM_CLASS (item_bar_parent_class)->update)(item, affine, clip_path, flags);
 
 	item->x1 = 0;
 	item->y1 = 0;
@@ -126,11 +153,14 @@ get_row_name (int n)
 }
 
 static void
-bar_draw_cell (ItemBar *item_bar, GdkDrawable *drawable, ItemBarSelectionType type,
-	       const char *str, int x1, int y1, int x2, int y2)
+bar_draw_cell (ItemBar const * const item_bar,
+	       GdkDrawable *drawable, ItemBarSelectionType const type,
+	       char const * const str,
+	       int const x, int const y,
+	       int const width, int const height)
 {
 	GtkWidget *canvas = GTK_WIDGET (GNOME_CANVAS_ITEM (item_bar)->canvas);
-	GdkFont *font = canvas->style->font;
+	GdkFont *font;
 	GdkGC *gc;
 	int len, texth, shadow;
 
@@ -139,109 +169,109 @@ bar_draw_cell (ItemBar *item_bar, GdkDrawable *drawable, ItemBarSelectionType ty
 	case ITEM_BAR_NO_SELECTION:
 		shadow = GTK_SHADOW_OUT;
 		gc = canvas->style->bg_gc [GTK_STATE_ACTIVE];
+		font = style_font_gdk_font (item_bar->normal_font);
 		break;
 
 	case ITEM_BAR_PARTIAL_SELECTION:
 		shadow = GTK_SHADOW_OUT;
-		gc = canvas->style->bg_gc [GTK_STATE_ACTIVE];
-		font = style_font_gdk_font (gnumeric_default_bold_font);
+		gc = canvas->style->dark_gc [GTK_STATE_PRELIGHT];
+		font = style_font_gdk_font (item_bar->bold_font);
 		break;
 
 	case ITEM_BAR_FULL_SELECTION:
 		shadow = GTK_SHADOW_IN;
 		gc = canvas->style->dark_gc [GTK_STATE_NORMAL];
-		font = style_font_gdk_font (gnumeric_default_bold_font);
+		font = style_font_gdk_font (item_bar->bold_font);
 		break;
 	}
 
 	len = gdk_string_width (font, str);
 	texth = font->ascent + font->descent;
 
-	gdk_draw_rectangle (drawable, gc, TRUE, x1 + 1, y1 + 1, x2-x1-2, y2-y1-2);
+	gdk_draw_rectangle (drawable, gc, TRUE, x + 1, y + 1, width-2, height-2);
 	gtk_draw_shadow (canvas->style, drawable, GTK_STATE_NORMAL, shadow,
-			 x1, y1, x2-x1, y2-y1);
+			 x, y, width, height);
 	gdk_draw_string (drawable, font, item_bar->gc,
-			 x1 + ((x2 - x1) - len) / 2,
-			 y1 + ((y2 - y1) - texth) / 2 + font->ascent,
+			 x + (width - len) / 2,
+			 y + (height - texth) / 2 + font->ascent,
 			 str);
 }
 
 static void
 item_bar_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width, int height)
 {
-	ItemBar *item_bar = ITEM_BAR (item);
-	Sheet   *sheet = item_bar->sheet_view->sheet;
-	ColRowInfo *cri;
-	int element, total, pixels, limit;
-	const char *str;
+	ItemBar const * const item_bar = ITEM_BAR (item);
+	Sheet   const * const sheet = item_bar->sheet_view->sheet;
+	int element = item_bar->first_element;
+	int pixels;
 
-	element = item_bar->first_element;
+	if (item_bar->orientation == GTK_ORIENTATION_VERTICAL) {
+		int total = -y;
+		int const real_width = GTK_WIDGET (item->canvas)->allocation.width;
 
-	if (item_bar->orientation == GTK_ORIENTATION_VERTICAL)
-		limit = y + height;
-	else
-		limit = x + width;
-
-	total = 0;
-
-	do {
-		if (item_bar->orientation == GTK_ORIENTATION_VERTICAL){
-
-			if (element >= SHEET_MAX_ROWS){
+		do {
+			if (element >= SHEET_MAX_ROWS) {
 				GtkWidget *canvas = GTK_WIDGET (item->canvas);
 
 				gtk_draw_shadow (canvas->style, drawable,
 						 GTK_STATE_NORMAL, GTK_SHADOW_OUT,
-						 x, y, width, height);
+						 -x, y, real_width, height);
 				return;
 			}
-			cri = sheet_row_get_info (sheet, element);
-			if (item_bar->resize_pos == element)
-				pixels = item_bar->resize_width;
-			else
+			if (item_bar->resize_pos != element) {
+				ColRowInfo const *cri = sheet_row_get_info (sheet, element);
 				pixels = cri->pixels;
+			} else
+				pixels = item_bar->resize_width;
 
-			if (total + pixels >= y){
-				str = get_row_name (element);
+			total += pixels;
+			if (total >= 0) {
+				char const * const str = get_row_name (element);
 				bar_draw_cell (item_bar, drawable,
 					       sheet_row_selection_type (sheet, element),
-					       str, -x, 1 + total - y,
-					       GTK_WIDGET (item->canvas)->allocation.width - x,
-					       1 + total + pixels - y);
+					       str,
+					       -x, 1 + total - pixels,
+					       real_width, pixels);
 			}
-		} else {
-			if (element >= SHEET_MAX_COLS){
+			++element;
+		} while (total < height);
+	} else {
+		int total = -x;
+		int const real_height = GTK_WIDGET (item->canvas)->allocation.height;
+
+		do {
+			if (element >= SHEET_MAX_COLS) {
 				GtkWidget *canvas = GTK_WIDGET (item->canvas);
 
 				gtk_draw_shadow (canvas->style, drawable,
 						 GTK_STATE_NORMAL, GTK_SHADOW_OUT,
-						 x, y, width, height);
+						 x, -y, width, real_height);
 				return;
 			}
-			cri = sheet_col_get_info (sheet, element);
-			if (item_bar->resize_pos == element)
-				pixels = item_bar->resize_width;
-			else
+			if (item_bar->resize_pos != element) {
+				ColRowInfo const *cri = cri = sheet_col_get_info (sheet, element);
 				pixels = cri->pixels;
+			} else
+				pixels = item_bar->resize_width;
 
-			if (total + pixels >= x){
-				str = col_name (element);
+			total += pixels;
+			if (total >= 0) {
+				char const * const str = col_name (element);
 				bar_draw_cell (item_bar, drawable,
 					       sheet_col_selection_type (sheet, element),
-					       str, 1 + total - x, -y,
-					       1 + total + pixels - x,
-					       GTK_WIDGET (item->canvas)->allocation.height - y);
+					       str,
+					       1 + total - pixels, -y,
+					       pixels, real_height);
 			}
-		}
 
-		total += pixels;
-		element++;
-	} while (total < limit);
+			++element;
+		} while (total < width);
+	}
 }
 
 static double
 item_bar_point (GnomeCanvasItem *item, double x, double y, int cx, int cy,
-		 GnomeCanvasItem **actual_item)
+		GnomeCanvasItem **actual_item)
 {
 	*actual_item = item;
 	return 0.0;
@@ -264,10 +294,15 @@ is_pointer_on_division (ItemBar *item_bar, int pos, int *the_total, int *the_ele
 	sheet = item_bar->sheet_view->sheet;
 
 	for (i = item_bar->first_element; total < pos; i++){
-		if (item_bar->orientation == GTK_ORIENTATION_VERTICAL)
+		if (item_bar->orientation == GTK_ORIENTATION_VERTICAL){
+			if (i >= SHEET_MAX_ROWS)
+				return NULL;
 			cri = sheet_row_get_info (sheet, i);
-		else
+		} else {
+			if (i >= SHEET_MAX_COLS)
+				return NULL;
 			cri = sheet_col_get_info (sheet, i);
+		}
 
 		total += cri->pixels;
 		if ((total - 4 < pos) && (pos < total + 4)){
@@ -298,9 +333,9 @@ set_cursor (ItemBar *item_bar, int pos)
 		return;
 
 	if (is_pointer_on_division (item_bar, pos, NULL, NULL))
-		gdk_window_set_cursor(canvas->window, item_bar->change_cursor);
+		gdk_window_set_cursor (canvas->window, item_bar->change_cursor);
 	else
-		gdk_window_set_cursor(canvas->window, item_bar->normal_cursor);
+		gdk_window_set_cursor (canvas->window, item_bar->normal_cursor);
 }
 
 /*
@@ -384,10 +419,15 @@ get_col_from_pos (ItemBar *item_bar, int pos)
 	total = 0;
 	sheet = item_bar->sheet_view->sheet;
 	for (i = item_bar->first_element; total < pos; i++){
-		if (item_bar->orientation == GTK_ORIENTATION_VERTICAL)
+		if (item_bar->orientation == GTK_ORIENTATION_VERTICAL){
+			if (i >= SHEET_MAX_ROWS)
+				return i-1;
 			cri = sheet_row_get_info (sheet, i);
-		else
+		} else {
+			if (i >= SHEET_MAX_COLS)
+				return i-1;
 			cri = sheet_col_get_info (sheet, i);
+		}
 
 		total += cri->pixels;
 		if (total > pos)
@@ -397,14 +437,22 @@ get_col_from_pos (ItemBar *item_bar, int pos)
 }
 
 static void
-colrow_tip_setlabel (ItemBar *item_bar, gboolean const is_vertical, int const size)
+colrow_tip_setlabel (ItemBar *item_bar, gboolean const is_vertical, int size)
 {
+	/*
+	 * FIXME : This measurement is off.
+	 * We need to take margins into account
+	 */
+	size -= 3;
+
 	if (item_bar->tip) {
 		char buffer [20 + sizeof (long) * 4];
 		if (is_vertical)
-			snprintf (buffer, sizeof (buffer), _("Height: %.2f"), size *.75);
+			snprintf (buffer, sizeof (buffer), _("Height: %.2f"),
+				  size * ROW_HEIGHT_SCALE);
 		else
-			snprintf (buffer, sizeof (buffer), _("Width: %.2f"), size / 7.5);
+			snprintf (buffer, sizeof (buffer), _("Width: %.2f"),
+				  size * COLUMN_WIDTH_SCALE);
 		gtk_label_set_text (GTK_LABEL (item_bar->tip), buffer);
 	}
 }
@@ -438,8 +486,8 @@ item_bar_event (GnomeCanvasItem *item, GdkEvent *e)
 	GnomeCanvas *canvas = item->canvas;
 	ItemBar *item_bar = ITEM_BAR (item);
 	int pos, start, element, x, y;
-	gboolean const resizing = ITEM_BAR_RESIZING (item_bar);
-	gboolean const is_vertical = (item_bar->orientation == GTK_ORIENTATION_VERTICAL);
+	const gboolean resizing = ITEM_BAR_RESIZING (item_bar);
+	const gboolean is_vertical = (item_bar->orientation == GTK_ORIENTATION_VERTICAL);
 
 	switch (e->type){
 	case GDK_ENTER_NOTIFY:
@@ -537,9 +585,9 @@ item_bar_event (GnomeCanvasItem *item, GdkEvent *e)
 						 element, e->button.state | GDK_BUTTON1_MASK);
 
 			if (is_vertical)
-				item_grid_popup_menu (sheet, e, 0, element);
+				item_grid_popup_menu (sheet, e, 0, element, FALSE, TRUE);
 			else
-				item_grid_popup_menu (sheet, e, element, 0);
+				item_grid_popup_menu (sheet, e, element, 0, TRUE, FALSE);
 		} else if (cri){
 			/*
 			 * Record the important bits.
@@ -631,6 +679,8 @@ item_bar_init (ItemBar *item_bar)
 	item_bar->resize_pos = -1;
 	item_bar->start_selection = -1;
 	item_bar->tip = NULL;
+	item_bar->normal_font = NULL;
+	item_bar->bold_font = NULL;
 }
 
 static void
@@ -670,7 +720,7 @@ item_bar_class_init (ItemBarClass *item_bar_class)
 	GtkObjectClass  *object_class;
 	GnomeCanvasItemClass *item_class;
 
-	item_bar_parent_class = gtk_type_class (gnome_canvas_item_get_type());
+	item_bar_parent_class = gtk_type_class (gnome_canvas_item_get_type ());
 
 	object_class = (GtkObjectClass *) item_bar_class;
 	item_class = (GnomeCanvasItemClass *) item_bar_class;
