@@ -16,6 +16,7 @@
 #include "number-match.h"
 #include "dump.h"
 #include "main.h"
+#include "application.h"
 #include "global-gnome-font.h"
 
 #include "../plugins/excel/boot.h"
@@ -28,6 +29,7 @@
 
 /* The debugging level */
 int gnumeric_debugging = 0;
+int style_debugging = 0;
 extern int ms_excel_read_debug;
 extern int ms_excel_formula_debug;
 extern int ms_excel_color_debug;
@@ -45,6 +47,8 @@ const struct poptOption gnumeric_popt_options [] = {
 	  N_("Dumps the function definitions"),   N_("FILE") },
 	{ "debug", '\0', POPT_ARG_INT, &gnumeric_debugging, 0,
 	  N_("Enables some debugging functions"), N_("LEVEL") },
+	{ "debug_styles", '\0', POPT_ARG_INT, &style_debugging, 0,
+	  N_("Enables some style related debugging functions"), N_("LEVEL") },
 
 	{ "debug_excel_read", '\0', POPT_ARG_INT,
 	    &ms_excel_read_debug, 0,
@@ -72,13 +76,16 @@ const struct poptOption gnumeric_popt_options [] = {
 static void
 gnumeric_main (void *closure, int argc, char *argv [])
 {
+	gboolean opened_workbook = FALSE;
 	int i;
-	
+	Workbook *new_book;
+
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
 
 	gnumeric_arg_parse (argc, argv);
 
+	application_init ();
 	string_init ();
 	format_match_init ();
 	style_init ();
@@ -100,28 +107,41 @@ gnumeric_main (void *closure, int argc, char *argv [])
 	glade_gnome_init ();
 	if (startup_glade_file)
 		glade_xml_new (startup_glade_file, NULL);
-	
+
 	if (dump_file_name){
 		dump_functions (dump_file_name);
 		exit (0);
 	}
 
+	/* Create an empty workbook, just in case we end up loading no files.
+	   NOTE: we need to create it here because workbook_do_destroy does
+	   special things if the number of workbooks reaches zero.  (And this
+	   would happen if loading the first file specified failed.)  */
+	new_book = workbook_new_with_sheets (1);
+
 	startup_files = poptGetArgs (ctx);
 	if (startup_files)
 		for (i = 0; startup_files [i]; i++) {
-			current_workbook = workbook_read (startup_files [i]);
-			
-			if (current_workbook)
-				gtk_widget_show (current_workbook->toplevel);
+			Workbook *new_book = workbook_read (startup_files [i]);
+
+			if (new_book) {
+				opened_workbook = TRUE;
+				gtk_widget_show (new_book->toplevel);
+			}
 
 			while (gtk_events_pending ()) /* Show something coherent */
 				gtk_main_iteration ();
 		}
 	poptFreeContext (ctx);
-	
-	if (current_workbook == NULL){
-		current_workbook = workbook_new_with_sheets (1);
-		gtk_widget_show (current_workbook->toplevel);
+
+	if (opened_workbook) {
+#ifdef ENABLE_BONOBO
+		gnome_object_unref (GNOME_OBJECT (new_book));
+#else
+		gtk_object_unref (GTK_OBJECT (new_book));
+#endif
+	} else {
+		gtk_widget_show (new_book->toplevel);
 	}
 
 #ifdef ENABLE_BONOBO
@@ -145,14 +165,14 @@ gnumeric_main (void *closure, int argc, char *argv [])
 int
 main (int argc, char *argv [])
 {
-	scm_boot_guile(argc, argv, gnumeric_main, 0);
+	scm_boot_guile (argc, argv, gnumeric_main, 0);
 	return 0;
 }
 #else
 int
 main (int argc, char *argv [])
 {
-	gnumeric_main(0, argc, argv);
+	gnumeric_main (0, argc, argv);
 	return 0;
 }
 #endif
