@@ -20,6 +20,11 @@
 #include "utils.h"
 #include "selection.h"
 
+/* Sizes for column/row headers */
+
+#define COLUMN_HEADER_HEIGHT 20
+#define ROW_HEADER_WIDTH 60
+
 static GtkTableClass *sheet_view_parent_class;
 
 void
@@ -133,10 +138,22 @@ sheet_view_set_zoom_factor (SheetView *sheet_view, double factor)
 	g_return_if_fail (sheet_view != NULL);
 	g_return_if_fail (IS_SHEET_VIEW (sheet_view));
 	
+	/* resize the header fonts */
+	item_bar_fonts_init (ITEM_BAR (sheet_view->col_item), factor);
+	item_bar_fonts_init (ITEM_BAR (sheet_view->row_item), factor);
+
 	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (sheet_view->sheet_view), factor);
-	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (sheet_view->col_canvas), factor);
-	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (sheet_view->row_canvas), factor);
 	
+	gtk_widget_set_usize (GTK_WIDGET (sheet_view->col_canvas),
+			      -1, COLUMN_HEADER_HEIGHT * factor);
+	gnome_canvas_set_scroll_region (GNOME_CANVAS (sheet_view->col_canvas), 0, 0,
+					1000000*factor, COLUMN_HEADER_HEIGHT*factor);
+
+	gtk_widget_set_usize (GTK_WIDGET (sheet_view->row_canvas),
+			      ROW_HEADER_WIDTH * factor, -1);
+	gnome_canvas_set_scroll_region (GNOME_CANVAS (sheet_view->row_canvas), 0, 0,
+					ROW_HEADER_WIDTH*factor, 1200000*factor);
+
 	gnome_canvas_scroll_to (GNOME_CANVAS (sheet_view->sheet_view), 0, 0);
 	gnome_canvas_scroll_to (GNOME_CANVAS (sheet_view->col_canvas), 0, 0);
 	gnome_canvas_scroll_to (GNOME_CANVAS (sheet_view->row_canvas), 0, 0);
@@ -165,12 +182,12 @@ new_canvas_bar (SheetView *sheet_view, GtkOrientation o, GnomeCanvasItem **itemp
 	/* FIXME: need to set the correct scrolling regions.  This will do for now */
 
 	if (o == GTK_ORIENTATION_VERTICAL){
-		w = 60;
+		w = ROW_HEADER_WIDTH;
 		h = -1;
-		gnome_canvas_set_scroll_region (GNOME_CANVAS (canvas), 0, 0, w, 1000000);
+		gnome_canvas_set_scroll_region (GNOME_CANVAS (canvas), 0, 0, w, 1200000);
 	} else {
 		w = -1;
-		h = 20;
+		h = COLUMN_HEADER_HEIGHT;
 		gnome_canvas_set_scroll_region (GNOME_CANVAS (canvas), 0, 0, 1000000, h);
 	}
 	gtk_widget_set_usize (canvas, w, h);
@@ -429,7 +446,6 @@ sheet_view_construct (SheetView *sheet_view)
 {
 	GnomeCanvasGroup *root_group;
 	GtkTable  *table = GTK_TABLE (sheet_view);
-	GtkWidget *select_all;
 	Sheet *sheet = sheet_view->sheet;
 	
 	/* Column canvas */
@@ -502,10 +518,10 @@ sheet_view_construct (SheetView *sheet_view)
 			NULL));
 	
 	/* The select-all button */
-	select_all = gtk_button_new ();
-	GTK_WIDGET_UNSET_FLAGS (select_all, GTK_CAN_FOCUS);
-	gtk_table_attach (table, select_all, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
-	gtk_signal_connect (GTK_OBJECT (select_all), "clicked",
+	sheet_view->select_all = gtk_button_new ();
+	GTK_WIDGET_UNSET_FLAGS (sheet_view->select_all, GTK_CAN_FOCUS);
+	gtk_table_attach (table, sheet_view->select_all, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_signal_connect (GTK_OBJECT (sheet_view->select_all), "clicked",
 			    GTK_SIGNAL_FUNC (button_select_all), sheet_view);
 	
 	/* Scroll bars and their adjustments */
