@@ -20,6 +20,7 @@
 #include "selection.h"
 #include "ranges.h"
 #include "print-info.h"
+#include "mstyle.h"
 #ifdef ENABLE_BONOBO
 #    include <libgnorba/gnorba.h>
 #endif
@@ -173,7 +174,8 @@ Sheet *
 sheet_new (Workbook *wb, const char *name)
 {
 	GtkWidget *sheet_view;
-	Sheet *sheet;
+	Sheet  *sheet;
+	MStyle *mstyle;
 
 	g_return_val_if_fail (wb != NULL, NULL);
 	g_return_val_if_fail (name != NULL, NULL);
@@ -182,21 +184,22 @@ sheet_new (Workbook *wb, const char *name)
 	sheet->signature = SHEET_SIGNATURE;
 	sheet->workbook = wb;
 	sheet->name = g_strdup (name);
+	sheet_create_styles (sheet);
 	sheet->last_zoom_factor_used = -1.0;
 	sheet->cols.max_used = -1;
 	sheet->rows.max_used = -1;
 
-	g_ptr_array_set_size (sheet->cols.info = g_ptr_array_new(), 
-			      COLROW_SEGMENT_INDEX(SHEET_MAX_COLS-1)+1);
-	g_ptr_array_set_size (sheet->rows.info = g_ptr_array_new(), 
-			      COLROW_SEGMENT_INDEX(SHEET_MAX_ROWS-1)+1);
+	g_ptr_array_set_size (sheet->cols.info = g_ptr_array_new (), 
+			      COLROW_SEGMENT_INDEX (SHEET_MAX_COLS-1)+1);
+	g_ptr_array_set_size (sheet->rows.info = g_ptr_array_new (), 
+			      COLROW_SEGMENT_INDEX (SHEET_MAX_ROWS-1)+1);
 	sheet->print_info = print_info_new ();
 
 	sheet->cell_hash = g_hash_table_new (cell_hash,
 					     (GCompareFunc)&cell_compare);
 
-	sheet->default_style = style_new ();
-	sheet_style_attach (sheet, 0, 0, SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1, sheet->default_style);
+	mstyle = mstyle_new_default ();
+	sheet_style_attach (sheet, sheet_get_full_range (), mstyle);
 
 	sheet_init_default_styles (sheet);
 
@@ -213,16 +216,10 @@ sheet_new (Workbook *wb, const char *name)
 	sheet_set_zoom_factor (sheet, 1.0);
 
 	sheet_corba_setup (sheet);
-	
-	return sheet;
-}
 
-static gboolean
-cell_hash_free_cell (gpointer	key, gpointer	value, gpointer	user_data)
-{
-	cell_destroy (value);
-	g_free (key);
-	return TRUE;
+	sheet_set_dirty (sheet, FALSE);
+
+	return sheet;
 }
 
 /**
@@ -252,8 +249,9 @@ sheet_foreach_colrow (Sheet *sheet, ColRowCollection *infos,
 
 	i = start;
 	while (i <= stop) {
-		ColRowInfo **segment = COLROW_GET_SEGMENT(infos, i);
+		ColRowInfo **segment = COLROW_GET_SEGMENT (infos, i);
 		int sub = COLROW_SUB_INDEX(i);
+
 		i += COLROW_SEGMENT_SIZE - sub;
 		if (segment != NULL)
 			for (; sub < COLROW_SEGMENT_SIZE; ++sub) {
@@ -276,30 +274,12 @@ sheet_compute_col_row_new_size (Sheet *sheet, ColRowInfo *ci, void *data)
 	 * Ensure that there is at least 1 pixel around every cell so that we
 	 * can mark the current cell
 	 */
-	ci->margin_a = MAX(ci->margin_a_pt * pix_per_unit, 1);
-	ci->margin_b = MAX(ci->margin_b_pt * pix_per_unit, 1);
+	ci->margin_a = MAX (ci->margin_a_pt * pix_per_unit, 1);
+	ci->margin_b = MAX (ci->margin_b_pt * pix_per_unit, 1);
 
 	return FALSE;
 }
 
-static Value *
-zoom_cell_style (Sheet *sheet, int col, int row, Cell *cell, void *user_data)
-{
-	StyleFont *sf;
-
-	/*
-	 * If the size is already set, skip
-	 */
-	if (cell->style->font->scale == sheet->last_zoom_factor_used)
-		return NULL;
-
-	sf = style_font_new_from (cell->style->font, sheet->last_zoom_factor_used);
-	cell_set_font_from_style (cell, sf);
-	style_font_unref (sf);
-
-	return NULL;
-}
-
 void
 sheet_set_zoom_factor (Sheet *sheet, double factor)
 {
@@ -334,27 +314,10 @@ sheet_set_zoom_factor (Sheet *sheet, double factor)
 	}
 
 	/*
-	 * Scale the fonts for every cell
+	 * FIXME: this slugs zoom performance and should not
+	 * be neccessary if we get rendering right IMHO.
 	 */
-	sheet_cell_foreach_range (
-		sheet, TRUE, 0, 0, SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1,
-		zoom_cell_style, sheet);
-
-	/*
-	 * Scale the internal font styles
-	 */
-	for (l = sheet->style_list; l; l = l->next){
-		StyleRegion *sr = l->data;
-		Style *style = sr->style;
-		StyleFont *scaled;
-		
-		if (!(style->valid_flags & STYLE_FONT))
-			continue;
-
-		scaled = style_font_new_from (style->font, factor);
-		style_font_unref (style->font);
-		style->font = scaled;
-	}
+	sheet_cells_update (sheet, sheet_get_full_range (), FALSE);
 }
 
 /*
@@ -379,7 +342,6 @@ sheet_row_new (Sheet *sheet)
 	g_return_val_if_fail (IS_SHEET (sheet), NULL);
 
 	*ri = sheet->rows.default_style;
-	row_init_span (ri);
 
 	return ri;
 }
@@ -401,7 +363,7 @@ void
 sheet_col_add (Sheet *sheet, ColRowInfo *cp)
 {
 	int const col = cp->pos;
-	ColRowInfo *** segment = (ColRowInfo ***)&COLROW_GET_SEGMENT(&(sheet->cols), col);
+	ColRowInfo *** segment = (ColRowInfo ***)&COLROW_GET_SEGMENT (&(sheet->cols), col);
 
 	g_return_if_fail (col >= 0);
 	g_return_if_fail (col < SHEET_MAX_COLS);
@@ -508,10 +470,10 @@ sheet_compute_visible_ranges (Sheet const *sheet)
 static void
 colrow_set_units (Sheet *sheet, ColRowInfo *info)
 {
-	double pix = sheet->last_zoom_factor_used;
+	double const pix = sheet->last_zoom_factor_used;
 
-	info->units  = (info->pixels -
-			(info->margin_a + info->margin_b + 1)) / pix;
+	info->units = (info->pixels -
+		       (info->margin_a + info->margin_b - 1)) / pix;
 }
 
 static void
@@ -550,7 +512,7 @@ sheet_row_info_set_height (Sheet *sheet, ColRowInfo *ri, int height, gboolean he
 	g_return_if_fail (ri != NULL);
 
 	if (height_set_by_user)
-		ri->hard_size = 1;
+		ri->hard_size = TRUE;
 
 	ri->pixels = height;
 	colrow_set_units (sheet, ri);
@@ -747,7 +709,11 @@ sheet_col_size_fit (Sheet *sheet, int col)
 	/* Reset to the default width if the column was empty */
 	if (max < 0)
 		max = sheet->cols.default_style.pixels;
-	return max + ci->margin_a + ci->margin_b;
+	else
+		/* No need to scale width by zoom factor, that was already done */
+		max += ci->margin_a + ci->margin_b;
+
+	return max;
 }
 
 /*
@@ -800,16 +766,20 @@ sheet_row_size_fit (Sheet *sheet, int row)
 	/* Reset to the default width if the column was empty */
 	if (max < 0)
 		max = sheet->rows.default_style.pixels;
+	else
+		/* No need to scale height by zoom factor, that was already done */
+		max += ri->margin_a + ri->margin_b;
 
-	return max + ri->margin_a + ri->margin_b;
+	return max;
 }
 
 typedef struct {
 	int col;
 	GList * cells;
 } closure_cells_in_col;
+
 static gboolean
-cb_collect_cells_in_col(Sheet *sheet, ColRowInfo *ri, closure_cells_in_col *dat)
+cb_collect_cells_in_col (Sheet *sheet, ColRowInfo *ri, closure_cells_in_col *dat)
 {
 	Cell * cell = row_cell_get_displayed_at (ri, dat->col);
 	if (cell)
@@ -1043,13 +1013,19 @@ static Value *
 cb_set_cell_value (Sheet *sheet, int col, int row, Cell *cell,
 		  void *user_data)
 {
-	closure_set_cell_value * info = user_data;
+	MStyle                 *mstyle;
+	closure_set_cell_value *info = user_data;
 
 	if (cell == NULL)
 		cell = sheet_cell_new (sheet, col, row);
 
-	if (!CELL_IS_FORMAT_SET (cell))
-		cell_set_format_simple (cell, info->format);
+	mstyle = mstyle_new ();
+	mstyle_set_format (mstyle, info->format);
+
+/*	if (!CELL_IS_FORMAT_SET (cell))
+	cell_set_format_simple (cell, info->format); */
+
+	sheet_style_attach_single (sheet, col, row, mstyle);
 	cell_set_value (cell, value_new_float (info->val));
 	return NULL;
 }
@@ -1073,7 +1049,7 @@ sheet_set_text (Sheet *sheet, char const *text, Range const * r)
 		if (end != text && *end == 0) {
 			/*
 			 * It is a number -- remain in General format.  Note
-			 * that we would other wise actually set a "0" format
+			 * that we would otherwise actually set a "0" format
 			 * for integers and that it would stick.
 			 */
 		} else if (format_match (text, &closure.val, &closure.format)) {
@@ -1201,7 +1177,7 @@ sheet_load_cell_val (Sheet *sheet)
 	entry = GTK_ENTRY (sheet->workbook->ea_input);
 	cell = sheet_cell_get (sheet, sheet->cursor_col, sheet->cursor_row);
 
-	if (cell){
+	if (cell) {
 		char *text;
 
 		text = cell_get_text (cell);
@@ -1260,68 +1236,6 @@ sheet_start_editing_at_cursor (Sheet *sheet, gboolean blankp, gboolean cursorp)
 	}
 }
 
-/****************************************************************************/
-
-typedef struct
-{
-	gboolean bold;
-	gboolean bold_common;
-
-	gboolean italic;
-	gboolean italic_common;
-
-	GnomeFont *font;
-	gboolean font_common;
-
-	double font_size;
-	gboolean font_size_common;
-
-	gboolean first;
-} range_homogeneous_style_p;
-
-static Value *
-cell_is_homogeneous (Sheet *sheet, int col, int row,
-		     Cell *cell, void *user_data)
-{
-	range_homogeneous_style_p *accum = user_data;
-
-	if (accum->first) {
-		accum->bold = cell->style->font->is_bold;
-		accum->italic = cell->style->font->is_italic;
-		accum->font = cell->style->font->font;
-		accum->first = FALSE;
-		accum->font_size = cell->style->font->size;
-	} else {
-		if (accum->italic != cell->style->font->is_italic)
-			accum->italic_common = FALSE;
-		if (accum->bold != cell->style->font->is_bold)
-			accum->bold_common = FALSE;
-		if (accum->font != cell->style->font->font)
-			accum->font_common = FALSE;
-
-		if ((accum->bold_common == FALSE) &&
-		    (accum->italic_common == FALSE) &&
-		    (accum->font_common == FALSE))
-			return value_terminate();
-	}
-	return NULL;
-}
-
-static void
-range_is_homogeneous(Sheet *sheet, 
-		     int start_col, int start_row,
-		     int end_col,   int end_row,
-		     void *closure)
-{
-	/*
-	 * FIXME : Only check existing cells for now.  When styles are
-	 * redone this will need rethinking.
-	 */
-	sheet_cell_foreach_range (sheet, TRUE,
-				  start_col, start_row, end_col, end_row,
-				  &cell_is_homogeneous, closure);
-}
-
 /**
  * sheet_update_controls:
  *
@@ -1331,53 +1245,16 @@ range_is_homogeneous(Sheet *sheet,
 void
 sheet_update_controls (Sheet *sheet)
 {
-	range_homogeneous_style_p closure;
-	int flags;
-	
-	memset (&closure, 0, sizeof (closure));
-	closure.first = TRUE;
-	closure.bold_common = TRUE;
-	closure.italic_common = TRUE;
-	closure.font_common = TRUE;
-	closure.font_size_common = TRUE;
+	/* FIXME: hack for now, should be 'cursor cell' of selection */
+	MStyle *mstyle;
 
-	/* Double counting is ok, don't bother breaking up the regions */
-	selection_apply (sheet, &range_is_homogeneous,
-			 TRUE, &closure);
+	g_return_if_fail (sheet != NULL);
 
-	if (closure.first) {
-		/*
-		 * If no cells are on the selection, use the first cell
-		 * in the range to compute the values
-		 */
-		SheetSelection const * const ss = sheet->selections->data;
-		Style *style = sheet_style_compute (sheet, ss->user.start.col,
-						    ss->user.start.row, NULL);
-		closure.bold = style->font->is_bold;
-		closure.italic = style->font->is_italic;
-		closure.font = style->font->font;
-		closure.font_size = style->font->size;
+	mstyle = sheet_style_compute (sheet, sheet->cursor_col,
+				      sheet->cursor_row);
 
-		flags = WORKBOOK_FEEDBACK_BOLD |
-			WORKBOOK_FEEDBACK_ITALIC |
-			WORKBOOK_FEEDBACK_FONT_SIZE |
-			WORKBOOK_FEEDBACK_FONT;
-		
-		style_destroy (style);
-	} else 
-		flags = (closure.bold_common ? WORKBOOK_FEEDBACK_BOLD : 0) |
-			(closure.italic_common ? WORKBOOK_FEEDBACK_ITALIC : 0) |
-			(closure.font_size_common ? WORKBOOK_FEEDBACK_FONT_SIZE : 0) |
-			(closure.font_common ? WORKBOOK_FEEDBACK_FONT : 0);
-	
-	if (flags == 0)
-		return;
-	
-	workbook_feedback_set (sheet->workbook, flags,
-			       closure.italic,
-			       closure.bold,
-			       closure.font_size,
-			       closure.font);
+	workbook_feedback_set (sheet->workbook, mstyle);
+	mstyle_unref (mstyle);
 }		
 
 int
@@ -1529,7 +1406,7 @@ int
 sheet_find_boundary_horizontal (Sheet *sheet, int start_col, int row,
 				int count, gboolean jump_to_boundaries)
 {
-	gboolean find_first = cell_is_blank(sheet_cell_get (sheet, start_col, row));
+	gboolean find_first = cell_is_blank (sheet_cell_get (sheet, start_col, row));
 	int new_col = start_col, prev_col = start_col;
 	gboolean keep_looking = FALSE;
 	int iterations = 0;
@@ -1549,7 +1426,7 @@ sheet_find_boundary_horizontal (Sheet *sheet, int start_col, int row,
 		else if (new_col >= SHEET_MAX_COLS)
 			new_col = SHEET_MAX_COLS-1;
 		else if (jump_to_boundaries) {
-			keep_looking = (cell_is_blank( sheet_cell_get (sheet, new_col, row)) == find_first);
+			keep_looking = (cell_is_blank (sheet_cell_get (sheet, new_col, row)) == find_first);
 			if (keep_looking)
 				prev_col = new_col;
 			else if (!find_first) {
@@ -1585,7 +1462,7 @@ int
 sheet_find_boundary_vertical (Sheet *sheet, int col, int start_row,
 			      int count, gboolean jump_to_boundaries)
 {
-	gboolean find_first = cell_is_blank(sheet_cell_get (sheet, col, start_row));
+	gboolean find_first = cell_is_blank (sheet_cell_get (sheet, col, start_row));
 	int new_row = start_row, prev_row = start_row;
 	gboolean keep_looking = FALSE;
 	int iterations = 0;
@@ -1604,7 +1481,7 @@ sheet_find_boundary_vertical (Sheet *sheet, int col, int start_row,
 		else if (new_row > SHEET_MAX_ROWS-1)
 			new_row = SHEET_MAX_ROWS-1;
 		else if (jump_to_boundaries) {
-			keep_looking = (cell_is_blank( sheet_cell_get (sheet, col, new_row)) == find_first);
+			keep_looking = (cell_is_blank (sheet_cell_get (sheet, col, new_row)) == find_first);
 			if (keep_looking)
 				prev_row = new_row;
 			else if (!find_first) {
@@ -1854,13 +1731,13 @@ sheet_col_get (Sheet const *sheet, int const pos)
 	g_return_val_if_fail (pos < SHEET_MAX_COLS, NULL);
 	g_return_val_if_fail (pos >= 0, NULL);
 
-	segment = COLROW_GET_SEGMENT(&(sheet->cols), pos);
+	segment = COLROW_GET_SEGMENT (&(sheet->cols), pos);
 	if (segment != NULL)
-		ci = segment[COLROW_SUB_INDEX(pos)];
+		ci = segment [COLROW_SUB_INDEX(pos)];
 	return ci;
 }
 /**
- * sheet_col_fetch:
+ * sheet_col_get:
  *
  * Returns an allocated column:  either an existing one, or a fresh copy
  */
@@ -1868,7 +1745,6 @@ ColRowInfo *
 sheet_col_fetch (Sheet *sheet, int pos)
 {
 	ColRowInfo * res = sheet_col_get (sheet, pos);
-
 	if (res == NULL)
 		if ((res = sheet_col_new (sheet)) != NULL) {
 			res->pos = pos;
@@ -1893,9 +1769,9 @@ sheet_row_get (Sheet const *sheet, int const pos)
 	g_return_val_if_fail (pos < SHEET_MAX_ROWS, NULL);
 	g_return_val_if_fail (pos >= 0, NULL);
 
-	segment = COLROW_GET_SEGMENT(&(sheet->rows), pos);
+	segment = COLROW_GET_SEGMENT (&(sheet->rows), pos);
 	if (segment != NULL)
-		ri = segment[COLROW_SUB_INDEX(pos)];
+		ri = segment [COLROW_SUB_INDEX(pos)];
 	return ri;
 }
 
@@ -2006,7 +1882,8 @@ sheet_cell_foreach_range (Sheet *sheet, int only_existing,
 	}
 
 	for (i = start_col; i <= end_col ; ++i) {
-		ColRowInfo *ci = sheet_col_get(sheet, i);
+		ColRowInfo *ci = sheet_col_get (sheet, i);
+
 		if (ci == NULL) {
 			if (!only_existing)
 				for (j = start_row; j <= end_row ; ++j) {
@@ -2019,7 +1896,7 @@ sheet_cell_foreach_range (Sheet *sheet, int only_existing,
 		}
 
 		for (j = start_row; j <= end_row ; ++j) {
-			ColRowInfo *ri = sheet_row_get(sheet, j);
+			ColRowInfo *ri = sheet_row_get (sheet, j);
 			Cell * cell = NULL;
 
 			if (ri != NULL)
@@ -2039,7 +1916,7 @@ static Value *
 fail_if_not_selected (Sheet *sheet, int col, int row, Cell *cell, void *user_data)
 {
 	if (!sheet_selection_is_cell_selected (sheet, col, row))
-		return value_terminate();
+		return value_terminate ();
 	else
 		return NULL;
 }
@@ -2085,6 +1962,11 @@ sheet_cell_add_to_hash (Sheet *sheet, Cell *cell)
 	Cell *cell_on_spot;
 	int left, right;
 
+	g_return_if_fail (cell->col != NULL);
+	g_return_if_fail (cell->col->pos < SHEET_MAX_COLS);
+	g_return_if_fail (cell->row != NULL);
+	g_return_if_fail (cell->row->pos < SHEET_MAX_ROWS);
+
 	/* See if another cell was displaying in our spot */
 	cell_on_spot = row_cell_get_displayed_at (cell->row, cell->col->pos);
 	if (cell_on_spot)
@@ -2117,15 +1999,6 @@ sheet_cell_add (Sheet *sheet, Cell *cell, int col, int row)
 	cell->row   = sheet_row_fetch (sheet, row);
 
 	cell_realize (cell);
-
-	if (!cell->style){
-		int flags;
-
-		cell->style = sheet_style_compute (sheet, col, row, &flags);
-
-		if (flags & STYLE_FORMAT)
-			cell->flags |= CELL_FORMAT_SET;
-	}
 	cell_calc_dimensions (cell);
 
 	sheet_cell_add_to_hash (sheet, cell);
@@ -2272,10 +2145,9 @@ sheet_cell_formula_unlink (Cell *cell)
  * Callback for sheet_cell_foreach_range to remove a set of cells.
  */
 static Value *
-cb_free_cell (Sheet *sheet, int col, int row, Cell *cell,
-	      void *user_data)
+cb_free_cell (Sheet *sheet, int col, int row, Cell *cell, void *user_data)
 {
-	sheet_cell_remove_from_hash (sheet, cell);
+	sheet_cell_remove_internal (sheet, cell);
 	cell_destroy (cell);
 	return NULL;
 }
@@ -2289,8 +2161,9 @@ static void
 sheet_col_destroy (Sheet *sheet, int const col, gboolean free_cells)
 {
 	ColRowInfo ***segment = (ColRowInfo ***)&COLROW_GET_SEGMENT(&(sheet->cols), col);
-	int const sub = COLROW_SUB_INDEX(col);
+	int const sub = COLROW_SUB_INDEX (col);
 	ColRowInfo *ci = NULL;
+
 	if (*segment == NULL)
 		return;
 	ci = (*segment)[sub];
@@ -2351,21 +2224,6 @@ sheet_row_destroy (Sheet *sheet, int const row, gboolean free_cells)
 	}
 }
 
-static void
-sheet_destroy_styles (Sheet *sheet)
-{
-	GList *l;
-
-	for (l = sheet->style_list; l; l = l->next){
-		StyleRegion *sr = l->data;
-
-		style_destroy (sr->style);
-		g_free (sr);
-	}
-	g_list_free (sheet->style_list);
-	sheet->style_list = NULL;
-}
-
 void
 sheet_destroy_contents (Sheet *sheet)
 {
@@ -2378,25 +2236,22 @@ sheet_destroy_contents (Sheet *sheet)
 
 	/* Clear the row spans 1st */
 	for (i = sheet->rows.max_used; i >= 0 ; --i)
-		row_destroy_span(sheet_row_get (sheet, i));
-
-	/* Free the cells in a random order */
-	g_hash_table_foreach_remove (sheet->cell_hash, &cell_hash_free_cell, NULL);
+		row_destroy_span (sheet_row_get (sheet, i));
 
 	/* Delete in ascending order to avoid decrementing max_used each time */
 	for (i = 0; i <= max_col; ++i)
-		sheet_col_destroy (sheet, i, FALSE);
+		sheet_col_destroy (sheet, i, TRUE);
 
 	for (i = 0; i <= max_row; ++i)
-		sheet_row_destroy (sheet, i, FALSE);
-
+		sheet_row_destroy (sheet, i, TRUE);
+	
 	/* Free segments too */
-	for (i = COLROW_SEGMENT_INDEX(max_col); i >= 0 ; --i)
+	for (i = COLROW_SEGMENT_INDEX (max_col); i >= 0 ; --i)
 		if ((tmp = g_ptr_array_index (sheet->cols.info, i)) != NULL) {
 			g_free (tmp);
 			g_ptr_array_index (sheet->cols.info, i) = NULL;
 		}
-	for (i = COLROW_SEGMENT_INDEX(max_row); i >= 0 ; --i)
+	for (i = COLROW_SEGMENT_INDEX (max_row); i >= 0 ; --i)
 		if ((tmp = g_ptr_array_index (sheet->rows.info, i)) != NULL) {
 			g_free (tmp);
 			g_ptr_array_index (sheet->rows.info, i) = NULL;
@@ -2425,13 +2280,6 @@ sheet_destroy (Sheet *sheet)
 		sheet->print_info = NULL;
 	}	
 
-	if (sheet->dependency_hash != NULL) {
-		if (g_hash_table_size (sheet->dependency_hash) != 0)
-			g_warning ("Dangling dependencies");
-		g_hash_table_destroy (sheet->dependency_hash);
-		sheet->dependency_hash = NULL;
-	}
-
 	if (sheet->objects) {
 		g_warning ("Reminder: need to destroy SheetObjects");
 	}
@@ -2447,10 +2295,19 @@ sheet_destroy (Sheet *sheet)
 	}
 	g_list_free (sheet->sheet_views);
 	sheet->sheet_views = NULL;
+	
 	g_list_free (sheet->comment_list);
 	sheet->comment_list = NULL;
 
 	sheet_destroy_contents (sheet);
+
+	if (sheet->dependency_hash != NULL) {
+		if (g_hash_table_size (sheet->dependency_hash) != 0)
+			g_warning ("Dangling dependencies");
+		g_hash_table_destroy (sheet->dependency_hash);
+		sheet->dependency_hash = NULL;
+	}
+
 	sheet_destroy_styles (sheet);
 
 	g_hash_table_destroy (sheet->cell_hash);
@@ -2467,8 +2324,8 @@ sheet_destroy (Sheet *sheet)
 
 struct sheet_clear_region_callback_data
 {
-	int start_col, start_row, end_col, end_row;
-	GList *l;
+	Range	 r;
+	GList	*l;
 };
 
 /*
@@ -2492,14 +2349,14 @@ assemble_clear_cell_list (Sheet *sheet, int col, int row, Cell *cell,
 	/* Flag an attempt to delete a subset of an array */
 	if (cell->parsed_node && cell->parsed_node->oper == OPER_ARRAY){
 		ArrayRef * ref = &cell->parsed_node->u.array;
-		if ((col - ref->x) < cb->start_col)
-			return value_terminate();
-		if ((row - ref->y) < cb->start_row)
-			return value_terminate();
-		if ((col - ref->x + ref->cols -1) > cb->end_col)
-			return value_terminate();
-		if ((row - ref->y + ref->rows -1) > cb->end_row)
-			return value_terminate();
+		if ((col - ref->x) < cb->r.start.col)
+			return value_terminate ();
+		if ((row - ref->y) < cb->r.start.row)
+			return value_terminate ();
+		if ((col - ref->x + ref->cols -1) > cb->r.end.col)
+			return value_terminate ();
+		if ((row - ref->y + ref->rows -1) > cb->r.end.row)
+			return value_terminate ();
 	}
 
 	cb->l = g_list_prepend (cb->l, cell);
@@ -2511,12 +2368,14 @@ assemble_clear_cell_list (Sheet *sheet, int col, int row, Cell *cell,
  *
  * Clears are region of cells
  *
+ * @keepStyles : If this is non-null then styles are not erased.
+ *
  * We assemble a list of cells to destroy, since we will be making changes
  * to the structure being manipulated by the sheet_cell_foreach_range routine
  */
 void
 sheet_clear_region (Sheet *sheet, int start_col, int start_row, int end_col, int end_row,
-		    void *closure)
+		    void *keepStyles)
 {
 	struct sheet_clear_region_callback_data cb;
 	GList *l;
@@ -2529,23 +2388,29 @@ sheet_clear_region (Sheet *sheet, int start_col, int start_row, int end_col, int
 	/* Queue a redraw for the cells being removed */
 	sheet_redraw_cell_region (sheet, start_col, start_row, end_col, end_row);
 
-	cb.start_col = start_col;
-	cb.start_row = start_row;
-	cb.end_col = end_col;
-	cb.end_row = end_row;
+	cb.r.start.col = start_col;
+	cb.r.start.row = start_row;
+	cb.r.end.col = end_col;
+	cb.r.end.row = end_row;
 	cb.l = NULL;
+
+	/* Clear the style in the region (new_default will ref the style for us). */
+	if (!keepStyles)
+		sheet_style_attach (sheet, cb.r, mstyle_new_default ());
+
 	if (sheet_cell_foreach_range (sheet, TRUE,
 				       start_col, start_row, end_col, end_row,
-				       assemble_clear_cell_list, &cb) == NULL){
+				       assemble_clear_cell_list, &cb) == NULL) {
 		cb.l = g_list_reverse (cb.l);
-		cell_freeze_redraws();
+		cell_freeze_redraws ();
+
 		for (l = cb.l; l; l = l->next){
 			Cell *cell = l->data;
 
 			sheet_cell_remove (sheet, cell);
 			cell_destroy (cell);
 		}
-		cell_thaw_redraws();
+		cell_thaw_redraws ();
 		workbook_recalc (sheet->workbook);
 	} else 
 		gnumeric_no_modify_array_notice (sheet->workbook);
@@ -2652,72 +2517,6 @@ sheet_clear_region_formats (Sheet *sheet, int start_col, int start_row, int end_
 		clear_cell_format, NULL);
 }
 
-void
-sheet_style_attach (Sheet *sheet, int start_col, int start_row, int end_col, int end_row, Style *style)
-{
-	StyleRegion *sr;
-
-	g_return_if_fail (sheet != NULL);
-	g_return_if_fail (IS_SHEET (sheet));
-	g_return_if_fail (style != NULL);
-	g_return_if_fail (start_col <= end_col);
-	g_return_if_fail (start_row <= end_row);
-
-	sr = g_new (StyleRegion, 1);
-	sr->range.start.col = start_col;
-	sr->range.start.row = start_row;
-	sr->range.end.col = end_col;
-	sr->range.end.row = end_row;
-	sr->style = style;
-
-	sheet->style_list = g_list_prepend (sheet->style_list, sr);
-}
-
-/**
- * sheet_style_compute:
- * @sheet:	 Which sheet we are looking up
- * @col:	 column
- * @row:	 row
- * @non_default: A pointer where we store the attributes
- *               the cell has which are not part of the
- *               default style.
- */
-Style *
-sheet_style_compute (Sheet const *sheet, int col, int row, int *non_default)
-{
-	GList *l;
-	Style *style;
-	g_return_val_if_fail (sheet != NULL, NULL);
-	g_return_val_if_fail (IS_SHEET (sheet), NULL);
-
-	style = style_new_empty ();
-
-	/* Look in the styles applied to the sheet */
-	for (l = sheet->style_list; l; l = l->next){
-		StyleRegion *sr = l->data;
-		int is_default_style = l->next == NULL;
-		int flags;
-
-		flags = style->valid_flags;
-
-		if (range_contains (&sr->range, col, row)) {
-			style_merge_to (style, sr->style);
-			if (style->valid_flags == STYLE_ALL){
-				if (non_default){
-					if (!is_default_style)
-						*non_default = STYLE_ALL;
-					else
-						*non_default = flags;
-				}
-				return style;
-			}
-		}
-	}
-
-	g_warning ("Strange, no style available here\n");
-	return style_new ();
-}
-
 void
 sheet_make_cell_visible (Sheet *sheet, int col, int row)
 {
@@ -2870,12 +2669,13 @@ sheet_fill_selection_with (Sheet *sheet, const char *str,
 		g_return_if_fail (str != NULL);
 	}
 
-	cell_freeze_redraws();
+	cell_freeze_redraws ();
 	for (; l; l = l->next){
 		SheetSelection *ss = l->data;
 		sheet_set_text (sheet, str, &ss->user);
 	}
-	cell_thaw_redraws();
+	cell_thaw_redraws ();
+	workbook_recalc (sheet->workbook);
 }
 
 void
@@ -2904,14 +2704,15 @@ sheet_show_cursor (Sheet *sheet)
 
 /* Can remove sheet since local references have NULL sheet */
 char *
-cellref_name (CellRef *cell_ref, int eval_col, int eval_row)
+cellref_name (CellRef *cell_ref, ParsePosition const *pp)
 {
 	static char buffer [sizeof (long) * 4 + 4];
 	char *p = buffer;
 	int col, row;
+	Sheet *sheet = cell_ref->sheet;
 
 	if (cell_ref->col_relative)
-		col = eval_col + cell_ref->col;
+		col = pp->col + cell_ref->col;
 	else {
 		*p++ = '$';
 		col = cell_ref->col;
@@ -2927,7 +2728,7 @@ cellref_name (CellRef *cell_ref, int eval_col, int eval_row)
 		*p++ = b + 'A';
 	}
 	if (cell_ref->row_relative)
-		row = eval_row + cell_ref->row;
+		row = pp->row + cell_ref->row;
 	else {
 		*p++ = '$';
 		row = cell_ref->row;
@@ -2936,10 +2737,7 @@ cellref_name (CellRef *cell_ref, int eval_col, int eval_row)
 	sprintf (p, "%d", row+1);
 
 	/* If it is a non-local reference, add the path to the external sheet */
-	if (cell_ref->sheet == NULL)
-		return g_strdup (buffer);
-	else {
-		Sheet *sheet = cell_ref->sheet;
+	if (sheet != NULL) {
 		char *s;
 
 		if (strchr (sheet->name, ' '))
@@ -2947,8 +2745,14 @@ cellref_name (CellRef *cell_ref, int eval_col, int eval_row)
 		else
 			s = g_strconcat (sheet->name, "!", buffer, NULL);
 
+		if (sheet->workbook != pp->wb) {
+			char * n = g_strconcat ("[", sheet->workbook->filename, "]", s, NULL);
+			g_free (s);
+			s = n;
+		}
 		return s;
-	}
+	} else
+		return g_strdup (buffer);
 }
 
 gboolean
@@ -3202,7 +3006,8 @@ sheet_set_selection (Sheet *sheet, SheetSelection const *ss)
 
 /****************************************************************************/
 
-/* FIXME FIXME FIXME : 1999/Sept/29 when the style changes are done we need
+/*
+ * FIXME FIXME FIXME : 1999/Sept/29 when the style changes are done we need
  *  to figure out how row/col/cell movement will apply to that.
  */
 
@@ -3234,7 +3039,7 @@ avoid_dividing_array_horizontal (Sheet *sheet, int col, int row, Cell *cell,
 	    cell->parsed_node->oper != OPER_ARRAY ||
 	    cell->parsed_node->u.array.x <= 0)
 		return NULL;
-	return value_terminate();
+	return value_terminate ();
 }
 
 /*
@@ -3250,7 +3055,7 @@ avoid_dividing_array_vertical (Sheet *sheet, int col, int row, Cell *cell,
 	    cell->parsed_node->oper != OPER_ARRAY ||
 	    cell->parsed_node->u.array.y <= 0)
 		return NULL;
-	return value_terminate();
+	return value_terminate ();
 }
 
 /*
@@ -3290,7 +3095,7 @@ colrow_move (Sheet *sheet,
 	cells = g_list_reverse (cells);
 
 	/* Update the position */
-	segment[COLROW_SUB_INDEX(old_pos)] = NULL;
+	segment [COLROW_SUB_INDEX (old_pos)] = NULL;
 	info->pos = new_pos;
 
 	/* TODO : Figure out a way to merge these functions */
@@ -3362,14 +3167,17 @@ sheet_insert_col (Sheet *sheet, int col, int count)
 		colrow_move (sheet, i, 0, i, SHEET_MAX_ROWS-1,
 			     &sheet->cols, i, i + count);
 
-	/* 4. Recompute dependencies */
+	/* 4. Slide all the StyleRegion's right */
+	sheet_style_insert_colrow (sheet, col, count, TRUE);
+
+	/* 5. Recompute dependencies */
 	deps = region_get_dependencies (sheet,
 					col, 0,
 					SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1);
 	cell_queue_recalc_list (deps, TRUE);
 	workbook_recalc (sheet->workbook);
 
-	/* 5. Redraw */
+	/* 6. Redraw */
 	sheet_redraw_all (sheet);
 }
 
@@ -3428,18 +3236,20 @@ sheet_delete_col (Sheet *sheet, int col, int count)
 		colrow_move (sheet, i, 0, i, SHEET_MAX_ROWS-1,
 			     &sheet->cols, i, i-count);
 
-	/* 5. Recompute dependencies */
+	/* 5. Slide all the StyleRegion's up */
+	sheet_style_delete_colrow (sheet, col, count, TRUE);
+
+	/* 6. Recompute dependencies */
 	deps = region_get_dependencies (sheet,
 					col, 0,
 					SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1);
 	cell_queue_recalc_list (deps, TRUE);
 	workbook_recalc (sheet->workbook);
 
-	/* 6. Redraw */
+	/* 7. Redraw */
 	sheet_redraw_all (sheet);
 }
 
-///////////////////////////////////////
 /**
  * sheet_insert_row:
  * @sheet   The sheet
@@ -3495,14 +3305,17 @@ sheet_insert_row (Sheet *sheet, int row, int count)
 		colrow_move (sheet, 0, i, SHEET_MAX_COLS-1, i,
 			     &sheet->rows, i, i+count);
 
-	/* 4. Recompute dependencies */
+	/* 4. Slide all the StyleRegion's right */
+	sheet_style_insert_colrow (sheet, row, count, FALSE);
+
+	/* 5. Recompute dependencies */
 	deps = region_get_dependencies (sheet,
 					0, row,
 					SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1);
 	cell_queue_recalc_list (deps, TRUE);
 	workbook_recalc (sheet->workbook);
 
-	/* 5. Redraw */
+	/* 6. Redraw */
 	sheet_redraw_all (sheet);
 }
 
@@ -3561,30 +3374,31 @@ sheet_delete_row (Sheet *sheet, int row, int count)
 		colrow_move (sheet, 0, i, SHEET_MAX_COLS-1, i,
 			     &sheet->rows, i, i-count);
 
-	/* 5. Recompute dependencies */
+	/* 5. Slide all the StyleRegion's up */
+	sheet_style_delete_colrow (sheet, row, count, FALSE);
+
+	/* 6. Recompute dependencies */
 	deps = region_get_dependencies (sheet,
 					0, row,
 					SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1);
 	cell_queue_recalc_list (deps, TRUE);
 	workbook_recalc (sheet->workbook);
 
-	/* 6. Redraw */
+	/* 7. Redraw */
 	sheet_redraw_all (sheet);
 }
 
 /*
- * This is currently only used by the shift routines.
- * However, it is intended to become the heart of the 'cut'
- * operation.
- *
  * Excel does not support paste special from a 'cut' will we want to
  * do that ?
  */
-static void
+void
 sheet_move_range (struct expr_relocate_info const * rinfo)
 {
 	GList *deps, *cells = NULL;
 	Cell  *cell;
+	gboolean inter_sheet_formula;
+	int	dummy;
 
 	g_return_if_fail (rinfo->origin_sheet != NULL);
 	g_return_if_fail (IS_SHEET (rinfo->origin_sheet));
@@ -3610,23 +3424,52 @@ sheet_move_range (struct expr_relocate_info const * rinfo)
 	 */
 	cells = g_list_reverse (cells);
 
+	/* 4. Clear the target area */
+	/* Pass a pointer, any pointer, rather than NULL. That flags
+	 * not to clear styles */
+	sheet_clear_region (rinfo->target_sheet, 
+			    rinfo->origin.start.col + rinfo->col_offset,
+			    rinfo->origin.start.row + rinfo->row_offset,
+			    rinfo->origin.end.col + rinfo->col_offset,
+			    rinfo->origin.end.row + rinfo->row_offset,
+			    &dummy);
+
 	/* Insert the cells back */
 	for (; cells != NULL ; cells = g_list_remove (cells, cell)) {
 		cell = cells->data;
 
-		/* Update the location */
-		cell->col = sheet_col_fetch (rinfo->target_sheet,
-					     cell->col->pos + rinfo->col_offset);
-		cell->row = sheet_row_fetch (rinfo->target_sheet,
-					     cell->row->pos + rinfo->row_offset);
+		/* check for out of bounds and delete if necessary */
+		if ((cell->col->pos + rinfo->col_offset) >= SHEET_MAX_COLS ||
+		    (cell->row->pos + rinfo->row_offset) >= SHEET_MAX_ROWS) {
+			if (cell->parsed_node)
+				sheet_cell_formula_unlink (cell);
+			cell_unrealize (cell);
+			cell_destroy (cell);
+			continue;
+		}
 
-		sheet_cell_add_to_hash (rinfo->target_sheet, cell);
+		/* Inter sheet movement requires the moving the formula too */
+		inter_sheet_formula  = (cell->sheet != rinfo->target_sheet
+					&& cell->parsed_node);
+		if (inter_sheet_formula)
+			sheet_cell_formula_unlink (cell);
+
+		/* Update the location */
+		sheet_cell_add (rinfo->target_sheet, cell,
+				cell->col->pos + rinfo->col_offset,
+				cell->row->pos + rinfo->row_offset);
+
+		if (inter_sheet_formula)
+			sheet_cell_formula_link (cell);
 
 		/* Move comments */
 		cell_relocate (cell);
 	}
 
-	/* 4. Recompute dependencies */
+	/* 5. Slide styles */
+	sheet_style_relocate (rinfo);
+
+	/* 6. Recompute dependencies */
 	/* TODO : What region needs this ? the source or the target ? */
 	deps = region_get_dependencies (rinfo->target_sheet,
 					0, 0,
@@ -3634,7 +3477,7 @@ sheet_move_range (struct expr_relocate_info const * rinfo)
 	cell_queue_recalc_list (deps, TRUE);
 	workbook_recalc (rinfo->target_sheet->workbook);
 
-	/* 6. Redraw */
+	/* 7. Redraw */
 	sheet_redraw_all (rinfo->target_sheet);
 }
 
@@ -3648,7 +3491,7 @@ sheet_move_range (struct expr_relocate_info const * rinfo)
  *		delete count columns, positive number will insert
  *		count columns.
  *
- * Takes the cells to the in the region (col,start_row):(MAX_COL,end_row)
+ * Takes the cells in the region (col,start_row):(MAX_COL,end_row)
  * and copies them @count units (possibly negative) to the right.
  */
 
@@ -3677,7 +3520,7 @@ sheet_shift_rows (Sheet *sheet, int col, int start_row, int end_row, int count)
  *		delete count rows, positive number will insert
  *		count rows.
  *
- * Takes the cells to the in the region (start_col,row):(end_col,MAX_ROW)
+ * Takes the cells in the region (start_col,row):(end_col,MAX_ROW)
  * and copies them @count units (possibly negative) downwards.
  */
 void
